=== modified file 'NEWS'
--- NEWS	2006-12-15 02:47:54 +0000
+++ NEWS	2006-12-18 15:37:29 +0000
@@ -2,6 +2,11 @@
 
   IMPROVEMENTS:
 
+    * New connection: ``bzr+http://`` which supports tunnelling the smart
+      protocol over an HTTP connection. If writing is enabled on the bzr
+      server, then you can write over the http connection.
+      (Andrew Bennetts, John Arbash Meinel)
+
     * Aliases now support quotation marks, so they can contain whitespace
       (Marius Kruger)
 
@@ -223,11 +228,6 @@
 
     * ``bzr help commands`` output is now shorter (Aaron Bentley)
 
-    * New connection: ``bzr+http://`` which supports tunnelling the smart
-      protocol over an HTTP connection. If writing is enabled on the bzr
-      server, then you can write over the http connection.
-      (Andrew Bennetts)
-
     * ``bzr`` now uses lazy importing to reduce the startup time. This has
       a moderate effect on lots of actions, especially ones that have
       little to do. For example ``bzr rocks`` time is down to 116ms from

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-10-13 09:00:52 +0000
+++ bzrlib/transport/__init__.py	2006-12-15 23:34:42 +0000
@@ -1193,6 +1193,9 @@
 register_lazy_transport('bzr://',
                         'bzrlib.transport.smart',
                         'SmartTCPTransport')
+register_lazy_transport('bzr+http://',
+                        'bzrlib.transport.smart',
+                        'SmartHTTPTransport')
 register_lazy_transport('bzr+ssh://',
                         'bzrlib.transport.smart',
                         'SmartSSHTransport')

=== modified file 'bzrlib/transport/http/wsgi.py'
--- bzrlib/transport/http/wsgi.py	2006-10-13 06:30:29 +0000
+++ bzrlib/transport/http/wsgi.py	2006-12-15 23:59:15 +0000
@@ -26,14 +26,18 @@
 from bzrlib.urlutils import local_path_to_url
     
 
-def make_app(root, prefix, path_var):
+def make_app(root, prefix, path_var, readonly=True):
     """Convenience function to construct a WSGI bzr smart server.
     
     :param root: a local path that requests will be relative to.
     :param prefix: See RelpathSetter.
     :param path_var: See RelpathSetter.
     """
-    base_transport = get_transport('readonly+' + local_path_to_url(root))
+    local_url = local_path_to_url(root)
+    if readonly:
+        base_transport = get_transport('readonly+' + local_url)
+    else:
+        base_transport = get_transport(local_url)
     app = SmartWSGIApp(base_transport)
     app = RelpathSetter(app, prefix, path_var)
     return app

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2006-10-17 13:03:59 +0000
+++ bzrlib/transport/smart.py	2006-12-18 15:34:08 +0000
@@ -218,7 +218,7 @@
     pass
 
 # must do this otherwise urllib can't parse the urls properly :(
-for scheme in ['ssh', 'bzr', 'bzr+loopback', 'bzr+ssh']:
+for scheme in ['ssh', 'bzr', 'bzr+loopback', 'bzr+ssh', 'bzr+http']:
     transport.register_urlparse_netloc_protocol(scheme)
 del scheme
 
@@ -1757,6 +1757,60 @@
         super(SmartSSHTransport, self).__init__(url, medium=medium)
 
 
+class SmartHTTPTransport(SmartTransport):
+    """Just a way to connect between a bzr+http:// url and http://.
+    
+    This connection operates slightly differently than the SmartSSHTransport.
+    It uses a plain http:// transport underneath, which defines what remote
+    .bzr/smart URL we are connected to. From there, all paths that are sent are
+    sent as relative paths, this way, the remote side can properly
+    de-reference them, since it is likely doing rewrite rules to translate an
+    HTTP path into a local path.
+    """
+
+    def __init__(self, url, http_transport=None):
+        assert url.startswith('bzr+http://')
+
+        if http_transport is None:
+            http_url = url[len('bzr+'):]
+            self._http_transport = transport.get_transport(http_url)
+        else:
+            self._http_transport = http_transport
+        http_medium = self._http_transport.get_smart_medium()
+        super(SmartHTTPTransport, self).__init__(url, medium=http_medium)
+
+    def _remote_path(self, relpath):
+        """After connecting HTTP Transport only deals in relative URLs."""
+        if relpath == '.':
+            return ''
+        else:
+            return relpath
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        
+        :param relpath: the relative path or path components
+        :type relpath: str or list
+        """
+        return self._unparse_url(self._combine_paths(self._path, relpath))
+
+    def clone(self, relative_url):
+        """Make a new SmartHTTPTransport related to me.
+
+        This is re-implemented rather than using the default
+        SmartTransport.clone() because we must be careful about the underlying
+        http transport.
+        """
+        if relative_url:
+            abs_url = self.abspath(relative_url)
+        else:
+            abs_url = self.base
+        # By cloning the underlying http_transport, we are able to share the
+        # connection.
+        new_transport = self._http_transport.clone(relative_url)
+        return SmartHTTPTransport(abs_url, http_transport=new_transport)
+
+
 def get_test_permutations():
     """Return (transport, server) permutations for testing."""
     ### We may need a little more test framework support to construct an

=== modified file 'doc/http_smart_server.txt'
--- doc/http_smart_server.txt	2006-12-13 05:57:11 +0000
+++ doc/http_smart_server.txt	2006-12-18 15:42:15 +0000
@@ -48,7 +48,7 @@
         Options Indexes, FollowSymLinks
         RewriteEngine On
         RewriteBase /code
-        RewriteRule ^(.*)/\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
+        RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
     </Directory>
     
     # bzr-smart.fcgi isn't under the DocumentRoot, so Alias it into the URL
@@ -81,11 +81,11 @@
 Define the rewrite rules with mod_rewrite the same way as for FastCGI, except
 change::
 
-    RewriteRule ^(.*)/\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
+    RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
 
 to::
 
-    RewriteRule ^(.*)/\.bzr/smart$ /srv/example.com/scripts/bzr-smart.py
+    RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.py
 
 Like with mod_fastcgi, we also define how our script is to be handled::
 
@@ -94,7 +94,7 @@
         <Files bzr-smart.py>
             PythonPath "sys.path+['/srv/example.com/scripts']"
             AddHandler python-program .py
-            PythonHandler bzr-smart
+            PythonHandler bzr-smart::handler
         </Files>
     </Directory>
 
@@ -123,7 +123,8 @@
     smart_server_app = wsgi.make_app(
         root='/srv/example.com/code',
         prefix='/code/',
-        path_var='REQUEST_URI')
+        path_var='REQUEST_URI',
+        readonly=True)
 
     fcgi.WSGIServer(smart_server_app).run()
         
@@ -146,9 +147,13 @@
     smart_server_app = wsgi.make_app(
         root='/srv/example.com/code',
         prefix='/code/',
-        path_var='REQUEST_URI')
+        path_var='REQUEST_URI',
+        readonly=True)
 
-    modpywsgi.WSGIServer(smart_server_app).run()
+    def handler(request):
+        """Handle a single request."""
+        wsgi_server = modpywsgi.WSGIServer(smart_server_app)
+        return wsgi_server.run(request)
         
 The `modpywsgi` module can be found at http://trac.pocoo.org/wiki/ModPyWsgi.  It
 is part of pocoo_.
@@ -193,3 +198,22 @@
 
 .. _WSGI standard: http://www.python.org/dev/peps/pep-0333/
 
+
+Pushing over ``bzr+http://``
+----------------------------
+
+It is possible to allow pushing data over the http smart server. The
+easiest way to do this, is to just supply ``readonly=False`` to the
+``wsgi.make_app()`` call. But be careful, because the smart protocol does
+not contain any Authentication. So if you enable write support, you will
+want to restrict access to ``.bzr/smart`` URLs to restrict who can
+actually write data on your system.  At this time, it is not possible to
+allow some people to have read-only access and others to have read-write
+access to the same urls. Because at the HTTP layer (which is doing the
+Authenticating), everything is just a POST request.  However, it would
+certainly be possible to have HTTPS require authentication and use a
+writable server, and plain HTTP allow read-only access.
+
+
+.. 
+   vim: ft=rst tw=74 et

